
//
// Copyright (C) 2003-2005 Archie L. Cobbs and others.
// All rights reserved.
// 
// This file is part of the JC virtual machine software.
// 
// JC is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation; either version 2.1 of the License,
// or (at your option) any later version.
// 
// JC is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
// Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with JC in the file "LGPL-2.1"; if not, write to:
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307, USA.
//
// $Id: ClassfileFinder.java,v 1.1.1.1 2004/02/20 05:15:19 archiecobbs Exp $
//

package org.dellroad.jc;

/**
 * Represents objects capable of retrieving class files.
 */
public interface ClassfileFinder {

	/**
	 * Retrieve the class file that is used to define the named class.
	 * This is a capability specific to the JC virtual machine.
	 *
	 * @param className Class name (with slashes, not dots)
	 * @throws ClassNotFoundException if class is not found
	 */
	public byte[] getClassfile(String className)
		throws ClassNotFoundException;

	/**
	 * Return the hash of the bytes that would be returned by
	 * {@link #getClassfile getClassfile()} with the same arguments.
	 * The hash of a class file is defined as the last 16 bytes
	 * of the MD5 of the class file.
	 *
	 * @param className Class name (with slashes, not dots)
	 * @throws ClassNotFoundException if class is not found
	 */
	public long getClassfileHash(String className)
		throws ClassNotFoundException;
}

