
//
// Copyright (C) 2003-2005 Archie L. Cobbs and others.
// All rights reserved.
// 
// This file is part of the JC virtual machine software.
// 
// JC is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation; either version 2.1 of the License,
// or (at your option) any later version.
// 
// JC is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
// Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with JC in the file "LGPL-2.1"; if not, write to:
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307, USA.
//
// $Id: JCFinder.java,v 1.1.1.1 2004/02/20 05:15:19 archiecobbs Exp $
//

package org.dellroad.jc;

/**
 * Gateway into the JC virtual machine for retrieving class files.
 */
public class JCFinder implements ClassfileFinder {

	private final ClassLoader loader;

	/**
	 * Create a new finder that uses the given class loader
	 * to pull in not-before-seen class files as necessary.
	 */
	public JCFinder(ClassLoader loader) {
		this.loader = loader;
	}

	public byte[] getClassfile(String className)
	    throws ClassNotFoundException {
		try {
			return getClassfile(className, loader);
		} catch (NoClassDefFoundError e) {
			throw new ClassNotFoundException(e.getMessage());
		}
	}

	public long getClassfileHash(String className)
	    throws ClassNotFoundException {
		try {
			return getClassfileHash(className, loader);
		} catch (NoClassDefFoundError e) {
			throw new ClassNotFoundException(e.getMessage());
		}
	}

	private static native byte[] getClassfile(String className,
	    ClassLoader loader);

	private static native long getClassfileHash(String className,
	    ClassLoader loader);
}

