
//
// Copyright (C) 2003-2005 Archie L. Cobbs and others.
// All rights reserved.
// 
// This file is part of the JC virtual machine software.
// 
// JC is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation; either version 2.1 of the License,
// or (at your option) any later version.
// 
// JC is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
// Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with JC in the file "LGPL-2.1"; if not, write to:
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307, USA.
//
// $Id: ObjectGenerator.java,v 1.1.1.1 2004/02/20 05:15:19 archiecobbs Exp $
//

package org.dellroad.jc;

import java.io.File;

/**
 * Interface implemented by classes that generate JC ELF object
 * files from raw class files.
 */
public interface ObjectGenerator {

	/**
	 * Generate the ELF object file for the named class
	 * and write it into the file.
	 *
	 * @param className Class name (with slashes, not dots)
	 * @param finder Object capable of retrieving class files
	 * @param file Where to put resulting ELF object
	 */
	public void generateObject(String className,
	    ClassfileFinder finder, File file) throws Exception;

	/**
	 * Determine if an ELF object file is valid for the named class
	 * and all other classes (available through the provided
	 * <code>finder</code>) on which the object file may depend.
	 *
	 * @param className Class name (with slashes, not dots)
	 * @param finder Class finder capable of retrieving class files
	 * @param file Where to put resulting ELF object
	 */
	public boolean objectIsValid(String className,
	    ClassfileFinder finder, File file) throws Exception;
}

