
//
// Copyright (C) 2003-2005 Archie L. Cobbs and others.
// All rights reserved.
// 
// This file is part of the JC virtual machine software.
// 
// JC is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation; either version 2.1 of the License,
// or (at your option) any later version.
// 
// JC is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
// Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with JC in the file "LGPL-2.1"; if not, write to:
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307, USA.
//
// $Id: ActiveUseCheckStmt.java,v 1.1 2004/12/24 21:55:56 archiecobbs Exp $
//

package org.dellroad.jc.cgen;

import java.util.Random;
import soot.*;
import soot.jimple.*;
import soot.jimple.internal.JInvokeStmt;

/**
 * Jimple statement that represents a class initialization check.
 * This is a hack, as we have to look like a normal Jimple stmt.
 */
public class ActiveUseCheckStmt extends JInvokeStmt {

	// Random prefix to avoid collisions
	private static final String PREFIX
	    = Long.toHexString(new Random().nextLong()
		+ new Object().hashCode());

	public ActiveUseCheckStmt(SootClass sc) {
		super(Jimple.v().newStaticInvokeExpr(getCheckMethod(),
		    StringConstant.v(PREFIX + "/" + sc.getName())));
	}

	public void toString(UnitPrinter up)
	{
	    up.literal(toString());
	}

	public String toString()
	{
	    return "activeusecheck " + getSootClass(this).getName();
	}

	public static SootClass getSootClass(Stmt stmt) {
		return Scene.v().getSootClass(getClassName(stmt));
	}

	private static SootMethod getCheckMethod() {
	    return Scene.v().getSootClass("java.lang.Class")
		.getMethod("java.lang.Class forName(java.lang.String)");
	}

	public static void insertActiveUseCheck(PatchingChain units,
	    Stmt stmt, SootClass cl) {
		ActiveUseCheckStmt check = new ActiveUseCheckStmt(cl);
		units.insertBefore(check, stmt);
		stmt.redirectJumpsToThisTo(check);
	}

	public static boolean isActiveUseCheck(Stmt stmt) {
		if (stmt instanceof ActiveUseCheckStmt)
			return true;
		return getClassName(stmt) != null;
	}

	private static String getClassName(Stmt stmt) {
		if (!(stmt instanceof InvokeStmt))
			return null;
		InvokeExpr expr = ((InvokeStmt)stmt).getInvokeExpr();
		if (!expr.getMethod().equals(getCheckMethod()))
			return null;
		Value arg = expr.getArg(0);
		if (!(arg instanceof StringConstant))
			return null;
		String s = ((StringConstant)arg).value;
		int i;
		if ((i = s.indexOf('/')) != PREFIX.length()
		    || !s.substring(0, i).equals(PREFIX))
		    	return null;
		return s.substring(PREFIX.length() + 1);
	}
}

