
//
// Copyright (C) 2003-2005 Archie L. Cobbs and others.
// All rights reserved.
// 
// This file is part of the JC virtual machine software.
// 
// JC is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation; either version 2.1 of the License,
// or (at your option) any later version.
// 
// JC is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
// Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with JC in the file "LGPL-2.1"; if not, write to:
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307, USA.
//
// $Id: ClassConstant.java,v 1.1 2004/07/13 03:36:20 archiecobbs Exp $
//

package org.dellroad.jc.cgen;

import java.util.Random;
import soot.jimple.StringConstant;

/**
 * Hack for representing class constants, which Soot doesn't yet support.
 * We fake them as string constants with a special random prefix.
 * This class provides methods for converting back and forth.
 */
public class ClassConstant {

	// Random prefix to avoid collisions
	private static final String PREFIX
	    = Long.toHexString(new Random().nextLong()
	      + new Object().hashCode());

	private ClassConstant() {
	}

	public static StringConstant v(String className) {
		if (className.length() == 0 || className.charAt(0) == '[') {
			throw new RuntimeException("bogus class name `"
			    + className + "'");
		}
		return StringConstant.v(PREFIX + "/" + className);
	}

	public static boolean isClassConstant(StringConstant c) {
		int i;
		return (i = c.value.indexOf('/')) == PREFIX.length()
		    && c.value.substring(0, i).equals(PREFIX);
	}

	public static String getClassName(StringConstant c) {
		return c.value.substring(PREFIX.length() + 1);
	}
}

