
//
// Copyright (C) 2003-2005 Archie L. Cobbs and others.
// All rights reserved.
// 
// This file is part of the JC virtual machine software.
// 
// JC is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation; either version 2.1 of the License,
// or (at your option) any later version.
// 
// JC is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
// Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with JC in the file "LGPL-2.1"; if not, write to:
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307, USA.
//
// $Id: CodeGenerator.java,v 1.2 2004/07/13 03:36:20 archiecobbs Exp $
//

package org.dellroad.jc.cgen;

import java.io.OutputStream;
import org.dellroad.jc.ClassfileFinder;

/**
 * Interface for objects that can generate C source code and header files
 * that conform to the JC C source file standard.
 */
public interface CodeGenerator {

	/**
	 * Generate the C header file for the given class.
	 *
	 * @param className name of the class who's header file we're generating
	 * @param finder a way to acquire Java class files
	 * @param output where to write the generated header file to
	 */
	public void generateH(String className, ClassfileFinder finder,
	    OutputStream output) throws Exception;

	/**
	 * Generate the C source file for the given class.
	 *
	 * @param className name of the class who's C file we're generating
	 * @param finder a way to acquire Java class files
	 * @param output where to write the generated C file to
	 */
	public void generateC(String className, ClassfileFinder finder,
	    OutputStream output) throws Exception;

	/**
	 * Reset state.
	 *
	 * <p>
	 * Currently only one CodeGenerator instance will be
	 * in use at a time. Before and after each use, this
	 * method is invoked. A good oportunity to free objects
	 * that are no longer needed, (re)initialize state, etc.
	 */
	public void reset();
}

