
//
// Copyright (C) 2003-2005 Archie L. Cobbs and others.
// All rights reserved.
// 
// This file is part of the JC virtual machine software.
// 
// JC is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation; either version 2.1 of the License,
// or (at your option) any later version.
// 
// JC is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
// Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with JC in the file "LGPL-2.1"; if not, write to:
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307, USA.
//
// $Id: MethodOptimizer.java,v 1.5 2004/12/19 21:01:03 archiecobbs Exp $
//

package org.dellroad.jc.cgen;

import java.util.List;
import java.util.Set;
import soot.SootMethod;
import soot.jimple.StmtBody;
import soot.toolkits.graph.CompleteUnitGraph;

/**
 * Interface for optimizing the Jimple code associated with a method.
 * An implementation of this interface is assocated with each
 * {@link SootCodeGenerator SootCodeGenerator}.
 */
public interface MethodOptimizer {

	/**
	 * Optimize the given method's body.
	 *
	 * <p>
	 * Depedencies should be added
	 * to <code>deps</code> when the class may not be explicitly
	 * referenced in the new body. For example, if a call to
	 * <code>Arrays.sort()</code> were inlined, then <code>Arrays</code>
	 * would need to be added as an explicit dependency, because the
	 * inlined code might not explicitly refer to the <code>Arrays</code>
	 * class.
	 *
	 * @param method method to optimize
	 * @param deps set to which to add any class file dependencies
	 *	which may not be reflected in the byte code. To add a
	 *	dependency, add the class' {@link soot.SootClass SootClass}
	 *	object.
	 * @return control flow graph of optimized method body
	 */
	public CompleteUnitGraph optimize(SootMethod method, Set deps);
}

