
//
// Copyright (C) 2003-2005 Archie L. Cobbs and others.
// All rights reserved.
// 
// This file is part of the JC virtual machine software.
// 
// JC is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation; either version 2.1 of the License,
// or (at your option) any later version.
// 
// JC is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
// Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with JC in the file "LGPL-2.1"; if not, write to:
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307, USA.
//
// $Id: NullCheckStmt.java,v 1.3 2004/07/16 15:11:32 archiecobbs Exp $
//

package org.dellroad.jc.cgen;

import soot.*;
import soot.jimple.*;
import soot.jimple.internal.JInvokeStmt;

/**
 * Jimple statement that represents a null pointer check.
 * This is a hack, as we have to look like a normal Jimple stmt.
 */
public class NullCheckStmt extends JInvokeStmt {

	public NullCheckStmt(Value value) {
		super(Jimple.v().newSpecialInvokeExpr(
		    (Local)value, getNullCheckMethod()));
	}

	public Value getNullCheckValue() {
	    return ((InstanceInvokeExpr)getInvokeExpr()).getBase();
	}

	public void toString(UnitPrinter up)
	{
	    up.literal("nullcheck ");
	    getNullCheckValue().toString(up);
	}

	public String toString()
	{
	    return "nullcheck " + getNullCheckValue();
	}

	// Use a method we know is always going to be available...
	public static SootMethod getNullCheckMethod() {
	    return Scene.v().getSootClass("java.lang.Object")
		.getMethodByName("getClass");
	}

	public static void insertNullCheck(PatchingChain units,
	    Stmt stmt, Value value) {
		NullCheckStmt nullCheck = new NullCheckStmt(value);
		units.insertBefore(nullCheck, stmt);
		stmt.redirectJumpsToThisTo(nullCheck);
	}

	public static boolean isNullCheck(Stmt stmt) {

		// Check the obvious first
		if (stmt instanceof NullCheckStmt)
			return true;

		// It seems our NullCheckStmt's get replicated
		// sometimes and turn into plain old InvokeStmt's
		if (stmt instanceof InvokeStmt
		    && stmt.getInvokeExpr().getMethod() == getNullCheckMethod())
			return true;
		return false;
	}
}

