
//
// Copyright (C) 2003-2005 Archie L. Cobbs and others.
// All rights reserved.
// 
// This file is part of the JC virtual machine software.
// 
// JC is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation; either version 2.1 of the License,
// or (at your option) any later version.
// 
// JC is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
// Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with JC in the file "LGPL-2.1"; if not, write to:
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307, USA.
//
// $Id: StmtTagCopierSwitch.java,v 1.2 2004/12/21 01:54:03 archiecobbs Exp $
//

package org.dellroad.jc.cgen;

import soot.jimple.*;

/**
 * Copies tags from Jimple Stmt's to ValueBoxes.
 * All tags are copied.
 */
public class StmtTagCopierSwitch extends AbstractStmtSwitch {

	public void caseInvokeStmt(InvokeStmt stmt) {
		stmt.getInvokeExprBox().addAllTagsOf(stmt);
	}

	public void caseAssignStmt(AssignStmt stmt) {
		stmt.getRightOpBox().addAllTagsOf(stmt);
		stmt.getLeftOpBox().addAllTagsOf(stmt);
	}

	public void caseIdentityStmt(IdentityStmt stmt) {
		stmt.getRightOpBox().addAllTagsOf(stmt);
	}

	public void caseEnterMonitorStmt(EnterMonitorStmt stmt) {
		stmt.getOpBox().addAllTagsOf(stmt);
	}

	public void caseExitMonitorStmt(ExitMonitorStmt stmt) {
		stmt.getOpBox().addAllTagsOf(stmt);
	}

	public void caseIfStmt(IfStmt stmt) {
		stmt.getConditionBox().addAllTagsOf(stmt);
	}

	public void caseLookupSwitchStmt(LookupSwitchStmt stmt) {
		stmt.getKeyBox().addAllTagsOf(stmt);
	}

	public void caseReturnStmt(ReturnStmt stmt) {
		stmt.getOpBox().addAllTagsOf(stmt);
	}

	public void caseTableSwitchStmt(TableSwitchStmt stmt) {
		stmt.getKeyBox().addAllTagsOf(stmt);
	}

	public void caseThrowStmt(ThrowStmt stmt) {
		stmt.getOpBox().addAllTagsOf(stmt);
	}
}

