
//
// Copyright (C) 2003-2005 Archie L. Cobbs and others.
// All rights reserved.
// 
// This file is part of the JC virtual machine software.
// 
// JC is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation; either version 2.1 of the License,
// or (at your option) any later version.
// 
// JC is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
// Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with JC in the file "LGPL-2.1"; if not, write to:
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307, USA.
//
// $Id: ActiveUseValueSwitch.java,v 1.2 2004/12/22 23:39:06 archiecobbs Exp $
//

package org.dellroad.jc.cgen.analysis;

import soot.jimple.*;
import soot.grimp.AbstractGrimpValueSwitch;
import soot.grimp.NewInvokeExpr;
import soot.toolkits.scalar.FlowSet;

/**
 * Determine the SootClass which will definitely be initialized after
 * evaluating the value, if any.
 */
public class ActiveUseValueSwitch extends AbstractGrimpValueSwitch {

	public void caseNewInvokeExpr(NewInvokeExpr v) {
		setResult(v.getMethod().getDeclaringClass());
	}

	public void caseSpecialInvokeExpr(SpecialInvokeExpr v) {
		setResult(v.getMethod().getDeclaringClass());
	}

	public void caseStaticInvokeExpr(StaticInvokeExpr v) {
		setResult(v.getMethod().getDeclaringClass());
	}

	public void caseVirtualInvokeExpr(VirtualInvokeExpr v) {
		setResult(v.getMethod().getDeclaringClass());
	}

	public void caseNewExpr(NewExpr v) {
		setResult(v.getBaseType().getSootClass());
	}

	public void caseInstanceFieldRef(InstanceFieldRef v) {
		setResult(v.getField().getDeclaringClass());
	}

	public void caseStaticFieldRef(StaticFieldRef v) {
		setResult(v.getField().getDeclaringClass());
	}
}

