
//
// Copyright (C) 2003-2005 Archie L. Cobbs and others.
// All rights reserved.
// 
// This file is part of the JC virtual machine software.
// 
// JC is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation; either version 2.1 of the License,
// or (at your option) any later version.
// 
// JC is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
// Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with JC in the file "LGPL-2.1"; if not, write to:
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307, USA.
//
// $Id: FinalizeDetector.java,v 1.1 2004/12/17 15:35:46 archiecobbs Exp $
//

package org.dellroad.jc.cgen.analysis;

import java.util.*;
import org.dellroad.jc.cgen.*;
import soot.*;
import soot.jimple.*;

/**
 * Instance of this class detect 'new' expressions of objects that
 * override finalize(). The result is either <code>Boolean.TRUE</code>
 * or <code>Boolean.FALSE</code>.
 */
public class FinalizeDetector extends AbstractJimpleValueSwitch {

	public void caseNewExpr(NewExpr v) {
		SootClass sc = v.getBaseType().getSootClass();
		SootMethod finalize = sc.getMethod("finalize",
		    Collections.EMPTY_LIST, VoidType.v());
		setResult(Boolean.valueOf(
		    !finalize.getDeclaringClass().getName()
		      .equals("java.lang.Object")));
	}

	public void defaultCase(Object obj) {
		setResult(Boolean.FALSE);
	}
}

