
//
// Copyright (C) 2003-2005 Archie L. Cobbs and others.
// All rights reserved.
// 
// This file is part of the JC virtual machine software.
// 
// JC is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation; either version 2.1 of the License,
// or (at your option) any later version.
// 
// JC is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
// Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with JC in the file "LGPL-2.1"; if not, write to:
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307, USA.
//
// $Id: TypeAnalysis.java,v 1.6 2005/03/19 22:18:20 archiecobbs Exp $
//

package org.dellroad.jc.cgen.analysis;

import java.util.*;
import org.dellroad.jc.cgen.Util;
import soot.*;
import soot.jimple.*;
import soot.toolkits.scalar.*;

/**
 * Determines the run-time type of a reference local as best we can based on
 * information provided by a {@link LocalDefs LocalDefs} object.
 */
public class TypeAnalysis {

	private final LocalDefs localDefs;

	private final AbstractJimpleValueSwitch vs
	    = new AbstractJimpleValueSwitch() {
		public void caseNewArrayExpr(NewArrayExpr v) {
			setResult(v.getType());
		}
		public void caseNewMultiArrayExpr(NewMultiArrayExpr v) {
			setResult(v.getType());
		}
		public void caseNewExpr(NewExpr v) {
			setResult(v.getType());
		}
		public void caseNullConstant(NullConstant v) {
			setResult(v.getType());
		}
		public void defaultCase(Object obj) {
			Value v = (Value)obj;
			Type type = v.getType();
			if (!(v.getType() instanceof RefType))
				return;
			SootClass c = ((RefType)type).getSootClass();
			if (!Util.isFinal(c))
				return;
			setResult(type);
		}
	};

	public TypeAnalysis(LocalDefs localDefs) {
		this.localDefs = localDefs;
	}

	/**
	 * Return the exact type of the reference local def if it's a reference
	 * type and the Java class is known exactly, else <code>null</code>.
	 */
	public RefLikeType getExactType(Local local, Unit unit) {
		RefLikeType type = null;
		for (Iterator i = localDefs.getDefsOfAt(local, unit).iterator();
		    i.hasNext(); ) {
			DefinitionStmt def = (DefinitionStmt)i.next();
			vs.setResult(null);
			def.getRightOp().apply(vs);
			RefLikeType type2 = (RefLikeType)vs.getResult();
			if (type2 == null)
				return null;
			if (type == null)
				type = type2;
			else if (!type.equals(type2))
				return null;
		}
		return type;
	}
}

