
//
// Copyright (C) 2003-2005 Archie L. Cobbs and others.
// All rights reserved.
// 
// This file is part of the JC virtual machine software.
// 
// JC is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation; either version 2.1 of the License,
// or (at your option) any later version.
// 
// JC is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
// Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with JC in the file "LGPL-2.1"; if not, write to:
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307, USA.
//
// $Id: EscapingValueSwitch.java,v 1.1 2004/11/27 23:11:04 archiecobbs Exp $
//

package org.dellroad.jc.cgen.escape;

import java.util.*;
import org.dellroad.jc.cgen.*;
import soot.*;
import soot.jimple.*;

/**
 * Checks for values where the local can escape. The result
 * is <code>Boolean.TRUE</code> if so, <code>Boolean.FALSE</code> if not,
 * or <code>null</code> if the local can only escape as the 'return value'
 * of the <code>Value</code> itself.
 */
public class EscapingValueSwitch extends AbstractJimpleValueSwitch {

	private Local local;

	public Local getLocal() {
		return local;
	}

	public void setLocal(Local local) {
		this.local = local;
	}

	public void caseCastExpr(CastExpr v) {
		v.getOp().apply(this);
	}

	public void caseLocal(Local v) {
		setResult(v.equals(local) ? null : Boolean.FALSE);
	}

	public void caseInterfaceInvokeExpr(InterfaceInvokeExpr expr) {
		caseInstanceInvokeExpr(expr);
	}

	public void caseSpecialInvokeExpr(SpecialInvokeExpr expr) {
		caseInstanceInvokeExpr(expr);
	}

	public void caseVirtualInvokeExpr(VirtualInvokeExpr expr) {
		caseInstanceInvokeExpr(expr);
	}

	public void caseStaticInvokeExpr(StaticInvokeExpr expr) {
		handleInvokeParameters(expr);
	}

	private void caseInstanceInvokeExpr(InstanceInvokeExpr expr) {
		expr.getBase().apply(this);
		if (getResult() != Boolean.FALSE) {
			setResult(Boolean.TRUE);
			return;
		}
		handleInvokeParameters(expr);
	}

	private void handleInvokeParameters(InvokeExpr expr) {
		int argCount = expr.getArgCount();
		for (int i = 0; i < argCount; i++) {
			expr.getArg(i).apply(this);
			if (getResult() != Boolean.FALSE) {
				setResult(Boolean.TRUE);
				return;
			}
		}
		setResult(Boolean.FALSE);
	}

	public void defaultCase(Object obj) {
		setResult(Boolean.FALSE);
	}
}

