
/*
 * Copyright (C) 2003-2005 Archie L. Cobbs and others.
 * All rights reserved.
 * 
 * This file is part of the JC virtual machine software.
 * 
 * JC is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License,
 * or (at your option) any later version.
 * 
 * JC is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with JC in the file "LGPL-2.1"; if not, write to:
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * $Id: arch_structures.h,v 1.2 2004/12/14 03:03:25 archiecobbs Exp $
 */

#ifndef _ARCH_STRUCTURES_H_
#define _ARCH_STRUCTURES_H_

/************************************************************************
 *				Overview				*
 ************************************************************************/

/*

This purpose of this file is to define architecture-specific structures.
Here is a summary of structures that need to be defined by this file or
an included file:

Stack Frames
------------

This file must define a '_jc_stack_frame' type, an opaque type
representing a stack frame, with the following properties:

    - A stack frame is always associated with a function invocation,
      i.e., there is a calling function and a called function.
    - A stack frame is cabable of providing the return address in the
      calling function, i.e., the address of the next instruction to
      execute in the calling function after the called function returns.
    - A stack frame does not need to remain valid after the calling
      function returns.
    - Two stack frames can be compared for equality, where equality means
      they refer to the same caller -> callee invocation on the stack.
    - There must be an 'invalid' value not equal to any other value.
    - It is possible to get the 'current' stack frame.

*/

/************************************************************************
 *			Architecture-specific structures		*
 ************************************************************************/

#if defined(__i386__)
#include "i386/i386_structures.h"
#elif defined(__sparc__)
#include "sparc/sparc_structures.h"
#elif defined(__alpha__)
#include "alpha/alpha_structures.h"
#elif defined(__ia64__)
#include "ia64/ia64_structures.h"
#elif defined(__powerpc__)
#include "powerpc/powerpc_structures.h"
#else
#error "Unsupported architecture for architecture-specific structures"
#endif

#endif	/* _ARCH_STRUCTURES_H_ */

