
/*
 * Copyright (C) 2003-2005 Archie L. Cobbs and others.
 * All rights reserved.
 * 
 * This file is part of the JC virtual machine software.
 * 
 * JC is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License,
 * or (at your option) any later version.
 * 
 * JC is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with JC in the file "LGPL-2.1"; if not, write to:
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * $Id: fatal.c,v 1.1.1.1 2004/02/20 05:15:34 archiecobbs Exp $
 */

#include "libjc.h"

void
_jc_fatal_error(_jc_jvm *vm, const char *fmt, ...)
{
	va_list args;

	va_start(args, fmt);
	_jc_fatal_error_v(vm, fmt, args);
	/* va_end(args); */
}

void
_jc_fatal_error_v(_jc_jvm *vm, const char *fmt, va_list args)
{
	int (*printer)(FILE *, const char *, va_list);
	void (*aborter)(void);

	/* Print message */
	if (vm != NULL) {
		printer = vm->vfprintf;
		aborter = vm->abort;
	} else {
		printer = vfprintf;
		aborter = abort;
	}
	(*printer)(stderr, "jc: ", args);
	(*printer)(stderr, fmt, args);
	(*printer)(stderr, "\n", args);

	/* Die */
	(*aborter)();

	/* Should never get here */
	fprintf(stderr, "jc: abort() function returned!\n");
	abort();
}

