
/*
 * Copyright (C) 2003-2005 Archie L. Cobbs and others.
 * All rights reserved.
 * 
 * This file is part of the JC virtual machine software.
 * 
 * JC is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License,
 * or (at your option) any later version.
 * 
 * JC is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with JC in the file "LGPL-2.1"; if not, write to:
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * $Id: mutex.c,v 1.2 2004/07/05 21:03:27 archiecobbs Exp $
 */

#include "libjc.h"

/*
 * Initialize a mutex
 *
 * If unsuccessful an exception is stored.
 */
jint
_jc_mutex_init(_jc_env *env, pthread_mutex_t *mutex)
{
	pthread_mutexattr_t attr;
	int error;

	/* Initialize mutex attributes */
	if ((error = pthread_mutexattr_init(&attr)) != 0) {
		_JC_EX_STORE(env, InternalError,
		    "%s: %s", "pthread_mutexattr_init", strerror(error));
		return JNI_ERR;
	}

#if NDEBUG
	/* Enable debug checks */
	if ((error = pthread_mutexattr_settype(&attr,
	    PTHREAD_MUTEX_ERRORCHECK)) != 0) {
		_JC_EX_STORE(env, InternalError,
		    "%s: %s", "pthread_mutexattr_settype", strerror(error));
		pthread_mutexattr_destroy(&attr);
		return JNI_ERR;
	}
#endif

	/* Initialize mutex */
	if ((error = pthread_mutex_init(mutex, &attr)) != 0) {
		_JC_EX_STORE(env, InternalError,
		    "%s: %s", "pthread_mutex_init", strerror(error));
		pthread_mutexattr_destroy(&attr);
		return JNI_ERR;
	}

	/* Clean up */
	pthread_mutexattr_destroy(&attr);
	return JNI_OK;
}

/*
 * Destroy a mutex
 */
void
_jc_mutex_destroy(pthread_mutex_t *mutex)
{
	int r;

	r = pthread_mutex_destroy(mutex);
	_JC_ASSERT(r == 0);
}

/*
 * Initialize a condition variable.
 *
 * If unsuccessful an exception is stored.
 */
jint
_jc_cond_init(_jc_env *env, pthread_cond_t *cond)
{
	int error;

	/* Initialize mutex attributes */
	if ((error = pthread_cond_init(cond, NULL)) != 0) {
		_JC_EX_STORE(env, InternalError,
		    "%s: %s", "pthread_cond_init", strerror(error));
		return JNI_ERR;
	}

	/* Done */
	return JNI_OK;
}

/*
 * Destroy a condition variable.
 */
void
_jc_cond_destroy(pthread_cond_t *cond)
{
	int r;

	r = pthread_cond_destroy(cond);
	_JC_ASSERT(r == 0);
}

