
/*
 * Copyright (C) 2003-2005 Archie L. Cobbs and others.
 * All rights reserved.
 * 
 * This file is part of the JC virtual machine software.
 * 
 * JC is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License,
 * or (at your option) any later version.
 * 
 * JC is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with JC in the file "LGPL-2.1"; if not, write to:
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * $Id: os_functions.c,v 1.1.1.1 2004/02/20 05:15:42 archiecobbs Exp $
 */

#include "libjc.h"

/*

This file contains O/S dependent (but architecture independent) functions.
The following functions must be defined here:

    int
    _jc_num_cpus(_jc_env *env)

	This function should return the number of CPU's. It is used
	to implement the Runtime.availableProcessors() method.

 */

/************************************************************************
 *				FreeBSD					*
 ************************************************************************/

#if defined(__FreeBSD__)

#include <sys/sysctl.h>

int
_jc_num_cpus(_jc_env *env)
{
	static const char *const node = "hw.ncpu";
	_jc_jvm *const vm = env->vm;
	int num;

        if (sysctlbyname(node, NULL, 0, &num, sizeof(num)) == -1) {
		_jc_eprintf(vm, "sysctl(%s): %s", node, strerror(errno));
		return 1;
	}
	return num;
}

/************************************************************************
 *				Linux					*
 ************************************************************************/

#elif defined(__linux__)

int
_jc_num_cpus(_jc_env *env)
{
	_jc_jvm *const vm = env->vm;
        static const char *const file = "/proc/cpuinfo";
	char buf[64];
	size_t len;
	FILE *fp;
	int num;
	int ch;

        if ((fp = fopen("/proc/cpuinfo", "r")) == NULL) {
		_jc_eprintf(vm, "%s: %s", file, strerror(errno));
		return 1;
	}
	for (num = 0; fgets(buf, sizeof(buf), fp) != NULL; ) {
		len = strlen(buf);
		if (len > 0 && buf[len - 1] != '\n')
			while ((ch = getc(fp)) != '\n' && ch != EOF);
		if (strncmp(buf, "processor", 9) == 0)
			num++;
	}
	fclose(fp);
	return num;
}

/************************************************************************
 *				Others					*
 ************************************************************************/

#else
#error "Unsupported operating system"
#endif

