
/*
 * Copyright (C) 2003-2005 Archie L. Cobbs and others.
 * All rights reserved.
 * 
 * This file is part of the JC virtual machine software.
 * 
 * JC is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License,
 * or (at your option) any later version.
 * 
 * JC is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with JC in the file "LGPL-2.1"; if not, write to:
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * $Id: prepare.c,v 1.3 2005/02/27 04:52:59 archiecobbs Exp $
 */

#include "libjc.h"

/*
 * Prepare a type.
 *
 * NOTE: This assumes the caller is handling synchronization issues.
 */
jint
_jc_prepare_type(_jc_env *env, _jc_type *type)
{
	jint status;

	/* Already prepared? */
	if (_JC_FLG_TEST(type, PREPARED)) {
		_JC_ASSERT(_JC_FLG_TEST(type, LOADED));
		_JC_ASSERT(_JC_FLG_TEST(type, VERIFIED));
		return JNI_OK;
	}

	/* Sanity check */
	_JC_ASSERT(!_JC_FLG_TEST(type, ARRAY));

	/* Verify type first */
	if (!_JC_FLG_TEST(type, VERIFIED)
	    && (status = _jc_verify_type(env, type)) != JNI_OK)
		return status;

	/* Nothing to do for us */

	/* Mark type as prepared */
	type->flags |= _JC_TYPE_PREPARED;

	/* Done */
	return JNI_OK;
}

